import json
import logging
import secrets

from injector import inject
from paho.mqtt import client as mqtt_client

from .client import StoreClient
from .influxdb import InfluxDbStore
from .. import config

log = logging.getLogger(__name__)


class CombinedStore(StoreClient):

    @inject
    def __init__(self, configuration: config.Config):
        log.info("create combined store")
        self.stores = [
            InfluxDbStore(configuration),
            QueueStore(configuration)
        ]

    def store(self, data):
        for store in self.stores:
            store.store(data)


class QueueStore(StoreClient):
    @inject
    def __init__(self, configuration: config.Config):
        self.qos = configuration.queue_qos

        client_id = f'klimalogger-mqtt-{configuration.client_host_name}-{secrets.randbelow(1000)}'

        def on_connect(client, userdata, flags, rc):
            if rc == 0:
                log.debug("Connected to MQTT Broker!")
            else:
                log.error("Failed to connect, return code %d", rc)

        try:
            self.client = mqtt_client.Client(client_id)
            # client.username_pw_set(username, password)
            self.client.on_connect = on_connect
            log.info("connect to host %s, port %d", configuration.queue_host, configuration.queue_port)
            self.client.connect(configuration.queue_host, configuration.queue_port)
            self.client.loop_start()
        except Exception:
            log.exception("could not create client")
            self.client = None

    def __del__(self):
        if self.client:
            self.client.disconnect()

    def store(self, data: dict):
        if self.client:
            topic = "klimalogger"
            for entry in data:
                json_message = {}
                json_message.update(entry["tags"])
                json_message["time"] = entry["time"]
                json_message.update(entry["fields"])
                message = json.dumps(json_message)
                log.info("write data (%d bytes) to topic %s", len(message), topic)
                result = self.client.publish(topic, payload=message, qos=self.qos, )
                status = result[0]
                if status != 0:
                    log.warning("Failed to send message to topic %s with status %s", topic, status)
        else:
            log.warning("client not available")
            raise RuntimeError("bla")



# -*- coding: utf8 -*-

import adafruit_sgp40
import board
import busio
from injector import singleton, inject

from klimalogger import DataBuilder
from klimalogger.measurement import Measurements


@singleton
class Sensor:
    name = "SGP40"
    priority = 3

    @inject
    def __init__(self, i2c_bus: busio.I2C):
        self.driver = adafruit_sgp40.SGP40(i2c_bus)

    def measure(self, data_builder: DataBuilder, measurements: Measurements) -> None:
        voc_index = None
        if measurements.temperature is not None and measurements.relative_humidity is not None:
            raw_gas = self.driver.measure_raw(
                temperature=measurements.temperature, relative_humidity=measurements.relative_humidity
            )
            voc_index = self.driver.measure_index(
                temperature=measurements.temperature, relative_humidity=measurements.relative_humidity)
        else:
            raw_gas = self.driver.raw

        data_builder.add(self.name, "raw gas", "", float(raw_gas))
        if voc_index is not None:
            data_builder.add(self.name, "VOC index", "", float(voc_index))



#!/usr/bin/env python3
# -*- coding: utf8 -*-

import argparse
import logging

import klimalogger

parser = argparse.ArgumentParser()
parser.add_argument("-v", "--verbose", dest="verbose", action="store_true", help="verbose output")
parser.add_argument("-d", "--debug", dest="debug", action="store_true", help="debug output")
parser.add_argument("--check", help="measure once and print the results", action="store_true")
parser.add_argument("--service", help="run as service performing periodic measurements", action="store_true")
args = parser.parse_args()

klimalogger.add_log_handler(klimalogger.logger.create_console_handler())
klimalogger.set_log_level(logging.WARN)
if args.debug:
    klimalogger.set_log_level(logging.DEBUG)
elif args.verbose:
    klimalogger.set_log_level(logging.INFO)

client = klimalogger.client()
if args.check:
    print(client.measure())
else:
    if args.service:
        client.measure_and_store_periodically()
    else:
        client.measure_and_store()
